#!/usr/bin/perl
# Copyright (C) 2004 Andreas Gohr <andi@splitbrain.org>
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.

#defines
my $RANGE = 16777215;
my $K      = 8;
my @PRIME = (29, 379, 911, 1303, 1571, 2549, 4643, 6871);
my $RUNS  = 100;

use Data::Dumper;
srand();
%NET1;
%NET2;
$|=1;

if($ARGV[0] eq 'single'){
  singlenet();
}elsif($ARGV[0] eq 'multi'){
  multinet();
}else{
  print <<EOF;
  Simulation of Prophet Address Allocation

  Usage: 
    prophet-test.pl [single|multi]

  Parameter:

    single  Simulate a single Network and assign IPs until collision
    multi   Simulate two independent Networks until a collision between
            the two nets happen

  Note:
    
    All simulations are repeated $RUNS times and the average number of
    assigned IPs is displayed. This can take a long time for single net
    simulations.
EOF
}

########################################################################
# Subs

sub bootstrap(){
  #bootstrap NET1
  my $ip = int(rand($RANGE));
  %NET1 = {};
  $NET1{$ip}{'index'}=0;
  $NET1{$ip}{'state'}=[1,1,1,1,1,1,1,1];
  print "NET1 bootstrapped with IP: $ip\n";
  #bootstrap NET1
  my $ip = int(rand($RANGE));
  %NET2 = {};
  $NET2{$ip}{'index'}=0;
  $NET2{$ip}{'state'}=[1,1,1,1,1,1,1,1];
  print "NET2 bootstrapped with IP: $ip\n";
}

sub multinet(){
  my $swap=1;
  my $count=0;
  for (my $j=0; $j<$RUNS;){
    bootstrap();
    my $ret=0;
    while($ret == 0){
      if($swap == 1){
        $ret = mkip(\%NET1,\%NET2);
      }else{
        $ret = mkip(\%NET2,\%NET1);
      }
      $swap *= -1;
    }
    if ($ret == 1){
      print "Collision inside single Net with ".scalar(keys(%NET1))." IPs\n";
    }
    if ($ret == 2){
      print "Collision between two Nets with ".scalar(keys(%NET1))." IPs\n";
      $count += scalar(keys(%NET1)); 
      $j++
    }
  }
  printf("%.2f IPs per run\n",$count/$RUNS);
}

sub singlenet(){
  my $count=0;
  for (my $j=0; $j<$RUNS; $j++){
    bootstrap();
    my $ret=0;
    while($ret == 0){
      $ret = mkip(\%NET1,\%NET2);
    }
    if ($ret == 1){
      print "Collision in own Net with ".scalar(keys(%NET1))." IPs\n";
      $count += scalar(keys(%NET1));
    }
  }
  printf("%.2f IPs per run\n",$count/$RUNS);
}

sub mkip(){
  my $net  = $_[0];
  my $net2 = $_[1];
  my $ip = (keys(%$net))[rand(int(scalar(keys(%$net)))-1)];
  my $address = 1;

  #generate new IP
  for($i=0; $i<$K; $i++){
    $address *= (($PRIME[$i])**($net->{$ip}{'state'}[$i]));
    $address %= $RANGE;
#printf("i: %.0f p: %.0f s:%0.f a: %.0f\n",$i, $PRIME[$i],$net->{$ip}{'state'}[$i],$address);

  }
  $address += $ip;
  $address %= $RANGE;
  $address += 1;

#  print "Host $ip generated IP: $address\n";

  #check for existance in own net
  if(defined($net->{$address})){
    return 1;
  }
  #check for existance in other net
  if(defined($net2->{$address})){
    return 2;
  }

  #increase state
  $net->{$ip}{'state'}[$net->{$ip}{'index'}]++;

  #assign new IP to net
  $net->{$address}{'index'} = ($net->{$ip}{'index'}+1) % $K;
  $net->{$address}{'state'} = [@{$net->{$ip}{'state'}}];

  return 0;
}
