/**
 * Copyright (C) 2004 Andreas Gohr <andi@splitbrain.org>
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

#ifndef __PROPHET_H
#define __PROPHET_H

#include <libnet.h>
#include <pcap.h>     

// new ARP opcodes
#define ARP_PROPHET_REQ 55
#define ARP_PROPHET_REP 56

// IP Addresssize
#define IP_ADDR_LEN     4

// how often try to get an Address?
#define RETRY 5

// this defines the numbers of primes used
#define K 8

// this is the Networkpart in decimal for the assigned ip addresses
#define BASENET 167772160

// defines how many IPs to may be set (how many in host part)
#define RANGE 16777215

// more readable:
#define TRUE 1
#define FALSE 0

// some primes to feed our function 
// from http://primes.utm.edu/lists/small/1000.txt
u_long _prime[K] = {29, 379, 911, 1303, 1571, 2549, 4643, 6871};

// global devicename
char* _device; 

// null ip and mac broadcast
u_char _hw_broadcast[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
u_char _ip_nil[4]       = {0x00, 0x00, 0x00, 0x00};

// Container to hold state, index and NID
struct prophet_states {
    u_long nid;                         // the network id
    int index;                          // which state field to modify
    u_long state[K];                    // state of function
};
typedef struct prophet_states prophet_s;

// ARP header
struct sniff_arp {
    u_short arp_ht;                     // hardware type
    u_short arp_pt;                     // protocol type
    u_char arp_hlen;                    // hardware address length
    u_char arp_plen;                    // protocol address length
    u_short arp_op;                     // operation
    u_char arp_sha[ETHER_ADDR_LEN];     // sender hardware address
    u_char arp_spa[IP_ADDR_LEN];        // sender protocol address
    u_char arp_tha[ETHER_ADDR_LEN];     // target hardware address
    u_char arp_tpa[IP_ADDR_LEN];        // target protocol address
};

// global state container
prophet_s _prophet;

// global libnet handle
libnet_t* _libnet;

//function forward declaration
libnet_t* get_libnethandle();
pcap_t* get_pcaphandle();
int send_ARP_request();
int send_ARP(u_int16_t opcode,
             u_char* src_hw, u_char* src_ip,
             u_char* dst_hw, u_char* dst_ip);
int capture();
int request_address();
int wait_for_requests();
int check_reply(const u_char *packet, struct pcap_pkthdr *hdr);
void check_request(u_char *user,const struct pcap_pkthdr* pkthdr,
                   const u_char* packet);
u_long calcIP();
u_long randIP();
void bootstrap();
int setIP(u_long ip);
void usage();

#endif

